package jeliot.parser;

import java.util.*;
import jeliot.parser.error.*;
import jeliot.lang.*;

/**
  * PClass represents a single java class.
  *
  * @author Pekka Uronen
  *
  * created         5.8.1999
  */
public class PClass extends PElement {

    private PModifiers modifiers;
    private String name;
    private PType superClass;
    private Vector interfaces;
    private Vector classBody;
    private Vector fields = new Vector();

    private DomesticType type = new DomesticType(this);

    /** Creates a new class declaration. Called by the parser. */
    PClass(PModifiers modifiers, String name, PType superClass,
            Vector interfaces, Vector classBody, int nameStart,
            int right) {

        super(nameStart, right);
        this.modifiers = modifiers;
        this.name = name;
        this.superClass = superClass;
        this.interfaces = interfaces;
        this.classBody = classBody;

        setParents(classBody);

        // check modifier legality
        if ( modifiers.hasModifier(PModifiers.NATIVE) ) {
            throw new SemanticException("A class can't be native.");
        }
        if ( modifiers.hasModifier(PModifiers.SYNCHRONIZED) ) {
            throw new SemanticException("A class can't be synchronized.");
        }
        if ( modifiers.hasModifier(PModifiers.TRANSIENT) ) {
            throw new SemanticException("A class can't be transient.");
        }
        if ( modifiers.hasModifier(PModifiers.VOLATILE) ) {
            throw new SemanticException("A class can't be volatile.");
        }
    }

    PModifiers getModifiers() {
        return modifiers;
    }

    public String getName() {
        return name;
    }
    
    public PVariableDeclarator getField(String name) {
        Enumeration enum = fields.elements();
        while (enum.hasMoreElements()) {
            PVariableDeclarator f = 
                    (PVariableDeclarator)enum.nextElement();
            if (f.getIdentifier().equals(name)) {
                return f;
            }
        }
        return null;
    }
    
    public void registerField(PVariableDeclarator field) {
        fields.addElement(field);
    }
    
    public void acceptVisitor(ProgramTreeVisitor visitor) throws SemanticException {
        // tell the visitor to handle this class
        visitor.visitClass(this);

        // recurse to all enclosed classes
        Enumeration enum = classBody.elements();
        while (enum.hasMoreElements()) {
            PClassMember pm = (PClassMember)enum.nextElement();
            pm.acceptVisitor(visitor);
        }
    }
    
    public void resolveTypes(NameSpace space) {
        Enumeration enum = classBody.elements();
        while (enum.hasMoreElements()) {
            Object e = enum.nextElement();
            if (e instanceof PMethod) {
                ((PMethod)e).resolveType(space, this);
            }
        }
    }

    public PMethod getMainMethod() {
        return (PMethod)classBody.elementAt(0);
    }

    public DomesticType getType() {
        return type;
    }
    
    /** Returns a pointer to the method in this class with signature 
      * best matching the given name and arguments.
      * If no matching pointer is found, or the invocation in ambiguous,
      * throws a semantic exception.
      */
    public PMethod getMethod(String name, Type[] argt) {
        PMethod match = null;
        Vector matches = null;
        Enumeration enum = classBody.elements();
        while (enum.hasMoreElements()) {
            PElement elem = (PElement)enum.nextElement();
            if (elem instanceof PMethod) {
                PMethod meth = (PMethod)elem;
                if (meth.matches(name, argt)) {
                    if (match != null) {
                        if (matches != null) {
                            matches = new Vector();
                            matches.addElement(match.getPointer());
                        }
                        matches.addElement(meth.getPointer());
                    }
                    match = meth;
                }
            }
        }
        if (match == null) {
            throw new NonexistentMethodException(name, argt);
        }
        if (matches != null) {
            // determine the method with best matching signature.
            throw new AmbiguousMethodException(matches);
        }
        return match;
    }
    
}
