package jeliot.parser;

import java.util.*;
import jeliot.parser.error.*;
import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.FeatureNotImplementedException;

/**
  *
  * @author Pekka Uronen
  *
  * created         24.8.1999
  */
public class PMethodInvocation extends PExpression {
    
    public static final int STATIC      = 1;
    public static final int NONVIRTUAL  = 2;
    public static final int SUPER       = 3;
    public static final int INTERFACE   = 4;
    public static final int VIRTUAL     = 5;
    
    private PName name;
    private PExpression primary;
    private String identifier;
    private Vector arguments;
    
    private ReferenceType compileTimeType;
    private MethodPointer method;
    private int invocationMode;
    
    /** This is set to true, if the invocation is of form Type.method().
      * In that case, the called method must be static.
      */
    private boolean staticInvocation;
    
    PMethodInvocation(PName name, Vector arguments, int right) {
        super(name.getLeftPos(), right);
        this.name = name;
        this.identifier = name.getHead();
        this.arguments = arguments;
    
        setParents(arguments);
    }
    
    PMethodInvocation(PExpression primary, String identifier,
            Vector arguments, int right) {
    
        super(primary.getLeftPos(), right);
        this.primary = primary;
        this.identifier = identifier;
        this.arguments = arguments;
        setParents(arguments);
    }
    
    public String getIdentifier() {
        return identifier;
    }
    
    public Type getTargetType() {
        return compileTimeType;
    }
    
    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterMethodInvocation(this);
        if (primary != null) {
            primary.acceptVisitor(visitor);
        }
        if (arguments != null) {
            Enumeration enum = arguments.elements();
            while (enum.hasMoreElements()) {
                PExpression e = (PExpression)enum.nextElement();
                e.acceptVisitor(visitor);
            }
        }
        visitor.leaveMethodInvocation(this);
    }
    
    /** Finds out the class or interface in which the method invoked by
      * this invocation is declared.  
      *
      * This algorithm is described in [GJS96:15.11].
      *
      *@param space A name space which is searched to determine corrent
      *     class.
      *@param context The class in which the invokation takes place.
      */
    private ReferenceType resolveClass(LocalVariableBundle locals,
            NameSpace space, PClass context) {
    
        Type pType;
        if (primary != null) {
            pType = primary.getType();
        }
        else { 
            // Form of the invocation is MethodName.
            
            if (!name.isQualified()) {
                // simple name. The class is the class in which this 
                // method is declared in.
                return compileTimeType = context.getType();
            }
            // qualified name
            PName left = name.getLeftHand();
            int classification = left.classify(space, context);
            System.err.println("Classification "+classification);
            if (classification == PName.TYPE) {
                // static invocation
                //  - should reap out interfaces
                ReferenceType p = 
                        space.findClass(left.toString());
                staticInvocation = true;
                if (p == null) {
                    throw new SemanticException(
                            "No class found:" + left);
                }
                return compileTimeType = p;
            }
            // invocation is of form x.method(), where x is an 
            // expression name.
            System.err.println("Left: "+left);
            this.primary = new FieldAccess(left);
            primary.setParent(this);
            pType = ((FieldAccess)primary).resolveType(
                    space, locals, context);
        }
        if (!(pType instanceof ReferenceType)) {
            throw new IllegalInvocationException(
                    pType, this);
        }
        return compileTimeType = (ReferenceType)pType;    
    }
    
    public void typeCheck(
            LocalVariableBundle locals,
            NameSpace space,
            PClass myClass) {
                
        // step 1 - determine class/interface
        resolveClass(locals, space, myClass);
        
        // step 2 - determine method signature
        Type[] argt = getArgumentTypes();
        MethodPointer meth;
        try {
            meth = compileTimeType.getMethod(identifier, argt);
        }
        catch (NonexistentMethodException e) {
            e.setInvocation(this);
            throw (e);
        }
               
        // step 3 - is the chosen method appropriate?
        boolean classStat = myClass.getModifiers().
                hasModifier(PModifiers.STATIC);
        boolean methodStat = meth.isStatic();
        
        setMethod(meth);
        setType(meth.getReturnType());
        
        if (classStat && !methodStat) {
            throw new SemanticException("Non-static method" +
                    " invocation in static method.");
        }
        if (staticInvocation && !methodStat) {
            throw new SemanticException("Can't invoke instance method as static method.");
        }

        if (!methodStat) {
            throw new FeatureNotImplementedException("Instance methods");
        }
    }
    
    public Enumeration getArguments() {
        return arguments == null ?
                null :
                arguments.elements();
    }
    
    public Type[] getArgumentTypes() {
        int n = arguments == null ?
                0 :
                arguments.size();
        Type[] argt = new Type[n];
        for (int i = 0; i < n; ++i) {
            argt[i] = ( (PExpression)arguments.elementAt(i) ).getType();
        }
        return argt;
    }
    
    public void setMethod(MethodPointer meth) {
        this.method = meth;
        
        if (meth.isStatic()) {
            invocationMode = STATIC;
        }
        else if (meth.isPrivate()) {
            invocationMode = NONVIRTUAL;
        }
     //   else if (superInvocation) {
     //       invocationMode = SUPER;
     //   }
        else if (compileTimeType.isInterface()) {
            invocationMode = INTERFACE;
        }
        else {
            invocationMode = VIRTUAL;
        }
    }
    
    public boolean isStaticInvocation() {
        return staticInvocation;
    }
    
    public void setInvocationMode(int mode) {
        this.invocationMode = mode;
    }
    
    public Value evaluate(Director director) {
        super.evaluate(director);
    
        if (primary != null) {
            // evaluate target reference
        }
        
        int n = (arguments == null) ? 0 : arguments.size();
        Value[] val = new Value[n];
        for (int i = 0; i < n; ++i) {
            PExpression arg = (PExpression)arguments.elementAt(i);
            val[i] = arg.evaluate(director);
        }
        if (invocationMode == STATIC) {
            Return ret = method.invoke(null, val, this, director);
            return  ret == null ? null : ret.getValue();
        }
        else {
            // not implemented
            // ---
        }
        return null;
           
    }
    
}
