package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;

/**
  * PVariableDeclaration represents a field or local variable declaration. It may
  * declare and initialize one or more variables. Each declared variable
  * is presented by its own PVariableDeclarator instance.
  *
  * @author Pekka Uronen
  *
  * created         5.8.1999
  */
public class PVariableDeclaration extends PStatement implements
        PClassMember {

    protected PModifiers modifiers;
    protected PType type;
    protected Vector declarators;
    private boolean local;
    
    /** Fields */
    public PVariableDeclaration(PModifiers modifiers, PType type,
            Vector declarators, int right) {

        this.modifiers = modifiers;
        this.type = type;
        this.declarators = declarators;

        setLeftPos(modifiers.getLeftPos());
        int typeLeft = type.getLeftPos();
        if (typeLeft < getLeftPos()) {
            setLeftPos(typeLeft);
        }
        setRightPos(right);
        
        // let the declarators know who is their master declaration
        setParents(declarators);
    }
    
    public boolean isLocal() {
        return local;
    }
    
    /** For creating local variables. */
    public PVariableDeclaration(PType type, Vector declarators,
            boolean isFinal, int left) {

        makeDefaultModifiers(isFinal, left, left + 4); // length of "final"
        this.type = type;
        this.declarators = declarators;

        setLeftPos(left);

        // let the declarators know who is their master declaration
        setParents(declarators);
        local = true;
    }
    
    public boolean isStatic() {
        return modifiers.hasModifier(PModifiers.STATIC);
    }
    
    private void makeDefaultModifiers(boolean isFinal, int left, int right) {
        try {
            this.modifiers = new PModifiers();
            if (isFinal) {
                modifiers.setModifier(PModifiers.FINAL, left, right);
            }
        }
        catch (SemanticException e) {
            e.printStackTrace();
            System.exit(0);
        }
    }

    /** Creates a formal parameter with given type.
      * Note that this kind of declaration has no references to the
      * declarator. It is only used as a context to determine the type and the
      * modifiers (only final is possible) of the variable.
      */
    protected PVariableDeclaration(PType type, PVariableDeclarator dec,
            int finalLeft) {
                
        this.type = type;
        makeDefaultModifiers(true, finalLeft, finalLeft + 4);
        dec.setParent(this);
        this.declarators = new Vector();
        this.declarators.addElement(dec);
        local = true;
    }

    protected PVariableDeclaration(PType type, PVariableDeclarator dec) {        
        this.type = type;
        makeDefaultModifiers(false, 0, 0);
        dec.setParent(this);
        this.declarators = new Vector();
        this.declarators.addElement(dec);
        local = true;
    }

    /** Makes a contect to given variable declarator. This method is used
      * to create formal parameters.
      */
    public static PVariableDeclaration makeContext(
            PVariableDeclarator dec, PType type,
            boolean isFinal, int finalLeft) {

        if (isFinal) {
            return new PVariableDeclaration(type, dec, finalLeft);
        }
        else {
            return new PVariableDeclaration(type, dec);
        }
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.visitVariableDeclaration(this);
        Enumeration enum = declarators.elements();
        while (enum.hasMoreElements()) {
            PVariableDeclarator vd = (PVariableDeclarator)enum.nextElement();
            vd.acceptVisitor(visitor);
        }   
    }

    public Type resolveType(NameSpace space, PClass context) {
        type.resolveType(space, context);
        Enumeration enum = declarators.elements();
        while (enum.hasMoreElements()) {
            PVariableDeclarator vd = 
                    (PVariableDeclarator)enum.nextElement();
            vd.resolveType();
        }  
        return getType();
    }

    int countVariables() {
        return declarators.size();
    }

    Type getType() {
        return type.getType();
    }

    void execute(Director director) {
        director.atStatement(this);
        Enumeration enum = declarators.elements();
        while (enum.hasMoreElements()) {
            PVariableDeclarator d = (PVariableDeclarator)enum.nextElement();
            d.declare(director);
        }
    }

    public String toString() {
        return type + " declaration";
    }
    
    public PModifiers getModifiers() {
        return modifiers;
    }
    
    public Enumeration getDeclarators() {
        return declarators.elements();
    }
    
        
}
