package jeliot.parser;

import java.util.*;

import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.parser.error.*;

/**
  * PVariableDeclarator represents the declaration of a single variable.
  * It may be a field, a local variable or a formal parameter.
  *
  * @author Pekka Uronen
  *
  * created         5.8.1999
  */
public class PVariableDeclarator extends PElement {

    private String id;
    private PExpression init;
    private Vector arrayInit;
    private int dimension;
    private Type type;

    private int aiLeft;
    private int aiRight;
    
    private LocalFieldPointer myPointer = new LocalFieldPointer(this);
    
    public PVariableDeclarator(String id, int left) {
        this.id = id;
        setLeftPos(left);
        setRightPos(left + id.length());
    }

    public void setArrayInitializer(Vector arrayInit,
            int aiLeft, int aiRight) {
                
        this.arrayInit = arrayInit;
        this.aiLeft = aiLeft;
        this.aiRight = aiRight;
    }

    void setInitializer(PExpression init) {
        this.init = init;
        init.setParent(this);
        setRightPos(init.getRightPos());
    }

    /** Increases the dimension of this declarator by one. This is used
      * when constructing C-style array declarations. */
    void incDimension() {
        ++dimension;
    }

    public String getIdentifier() {
        return id;
    }

    public void acceptVisitor(ProgramTreeVisitor visitor) {
        visitor.enterVariableDeclarator(this);
        if (init != null) {
            init.acceptVisitor(visitor);
        }
        visitor.leaveVariableDeclarator(this);
    }
    
    public PExpression getInitializer() {
        return init;
    }

    public Type getType() {
        return type;
    }

    public void resolveType() {
        Type dType = ( (PVariableDeclaration)getParent() ).getType();
        if (dimension == 0) {
            this.type = dType;
        }
        else {
            this.type = ArrayType.getType(dType, dimension);
        }
    }

    void declare(Director director) {
        director.atDeclarator(this);
    
        Variable var = director.declareVariable(this);
        if (init != null) {
            Scratch scratch = director.openScratch();
            Value val = init.evaluate(director);
            var.assign(val);
            Value casted = var.getValue(); 
            director.animateAssignment(var, val, casted, null);
            director.closeScratch(scratch);
        }
    }

    public String toString() {
        return type + " "+id;
    }
    
    public boolean isStatic() {
        return ((PVariableDeclaration)getParent()).getModifiers().
                hasModifier(PModifiers.STATIC);
    }
    
    public boolean isPrivate() {
        return  ((PVariableDeclaration)getParent()).getModifiers().
                hasModifier(PModifiers.PRIVATE);
    }
    
    public FieldPointer getPointer() {
        return myPointer;
    }
    
    public void typeCheck(NameSpace space, LocalVariableBundle locals,
            PClass context) {

        resolveType();

        PVariableDeclaration d = (PVariableDeclaration)getParent();
        boolean local = d.isLocal();
        if (local) {
            PVariableDeclarator decl;
            if ((decl = (PVariableDeclarator)locals.get(id)) != null) {
                throw new DoubleDeclarationException(decl, this);
            }
        }
        else {
            PVariableDeclarator field;
            if ((field = context.getField(getIdentifier())) != null) {
                throw new DoubleDeclarationException(field, this);
            }
        }
        if (init != null ) {
            // this throws exception, if assignment is illegal.
            if (!init.getType().mayBeAssignedTo(getType())) {
                throw new IllegalAssignmentException(this);
            }
        }
        if (local) {
            locals.put(this);
        }
        else{
            context.registerField(this);
        }
    }    
    
}
