package jeliot.parser;

import java.util.*;
import jeliot.lang.*;
import jeliot.theatre.*;
import jeliot.parser.error.*;
import jeliot.parser.error.IllegalArgumentException;
import jeliot.FeatureNotImplementedException;

/**
  * @author Pekka Uronen
  *
  * created 15.8.1999
  */
public class TypeChecker extends ProgramTreeVisitor {

    /** Holds the local variables currently accessible. */
    private LocalVariableBundle currentLocals;

    /** Holds the imported and declared classes. */
    private NameSpace space;

    private PStatement currentStatement;
    private PClass currentClass;
    private PMethod currentMethod;
    
    private boolean staticContext = true;

    public TypeChecker(NameSpace space) {
        this.space = space;
    }

    /** Sets the current class context variable.
      */
    public void visitClass(PClass c) {
        this.currentClass = c;
        c.resolveTypes(space);
    }

    /** Initializes the local variable context with the formal
      * parameters.
      */
    public void visitMethod(PMethod m) {
        // create a dictionary for local variables
        currentLocals = new LocalVariableBundle();
        staticContext = m.isStatic();
        currentMethod = m;
        m.setDeclaringClass(currentClass);

    }

    public void leaveMethod(PMethod m) {
        m.setMaximumCount(currentLocals.getMaximumCount());
    }

    /** Sets the current statement context. */
    public void visitStatement(PStatement s) {
        currentStatement = s;
    }

    public void enterBlock(PBlock b) {
        currentLocals.saveState();
    }

    public void leaveBlock(PBlock b) {
        currentLocals.resumeState();
    }

    public void enterWhile(PWhile w) {
        currentStatement = w;
    }

    /** Checks that the condition expression is boolean.
      */
    public void leaveWhile(PWhile w) {
        if ( w.getCheck().getType() != PrimitiveType.booleanTYPE ) {
            throw new SemanticException(
                    "Boolean required in while check.");
        }
    }

    public void enterFor(PFor f) {
        currentStatement = f;
    }

    /** Checks that the condition expression is boolean.
      */
    public void leaveFor(PFor f) {
        if ( f.getCheck().getType() != PrimitiveType.booleanTYPE ) {
            throw new SemanticException(
                    "Boolean required in for check.");
        }
    }

    public void enterDoWhile(PDoWhile w) {
        currentStatement = w;
    }

    /** Checks that the condition expression is boolean.
      */
    public void leaveDoWhile(PDoWhile w) {
        if ( w.getCheck().getType() != PrimitiveType.booleanTYPE ) {
            throw new SemanticException(
                    "Boolean required in do-while check.");
        }
    }

    /** Checks that the condition expression is boolean.
      */
    public void visitThen(PIfThenElse ite) {
        if ( ite.getCheck().getType() != PrimitiveType.booleanTYPE ) {
            throw new SemanticException("Boolean required in if check.");
        }
    }

    public void visitExpressionStatement(PExpressionStatement s) {
        currentStatement = s;
    }

    /** Checks that the assignment is legal. Throws exception if the
      * right hand side type is not assignment convertible to the left
      * hand type. Also sets the type of the expression.
      */
    public void leaveAssignmentExpression(PAssignmentExpr e) {
        Type lht = e.getLeftHand().getType();
        // this throws exception, if assignment is illegal.
        if (!e.getRightHand().getType().mayBeAssignedTo(lht)) {
            throw new IllegalAssignmentException(e);
        }
        e.setType(lht);
    }

    public void leaveBinaryExpression(PBinaryExpr e) {
        e.typeCheck(space, currentLocals, currentClass);
    }

    public void leaveUnaryExpression(PUnaryExpr e) {
        // Resolve type
        PExpression argument = e.getArgument();
        Type t = argument.getType();

        switch (e.getOperator()) {
            case (PUnaryExpr.PLUS):
            case (PUnaryExpr.MINUS):
                if (!t.isNumeric()) {
                    throw new IllegalArgumentException(e);
                }
                e.setType(((PrimitiveType)t).unaryNumericPromotion());
                break;
            case (PUnaryExpr.PLUSPLUS):
            case (PUnaryExpr.MINUSMINUS):
            case (PUnaryExpr.POSTPLUSPLUS):
            case (PUnaryExpr.POSTMINUSMINUS):
                if (!(argument instanceof PVariableAccessExpr) ||
                        !t.isNumeric()) {
                            
                    throw new IllegalArgumentException(e);
                }
                e.setType(((PrimitiveType)t).unaryNumericPromotion());
                break;
            case (PUnaryExpr.COMP):
                if (!t.isIntegral()) {
                    throw new IllegalArgumentException(e);
                }
                e.setType(((PrimitiveType)t).unaryNumericPromotion());
                break;
            case (PUnaryExpr.NOT):
                if (t != PrimitiveType.booleanTYPE) {
                    throw new IllegalArgumentException(e);
                }
                e.setType(PrimitiveType.booleanTYPE);
                break;
            default:
                throw new RuntimeException();
        }
    }

    public void leaveCastExpression(PCastExpression e) {
        e.typeCheck(space, currentLocals, currentClass);
    }

    /** */
    public void visitVariableAccess(PVariableAccessExpr e) {
        e.resolveType(space, currentLocals, currentClass,
                staticContext);
    }

    public void visitVariableDeclaration(PVariableDeclaration d) {
        d.resolveType(space, currentClass);
    }

    public void leaveVariableDeclarator(PVariableDeclarator l) {
        l.typeCheck(space, currentLocals, currentClass);
    }

    /** This algorithm is described in [GJS96:15.11].
      */
    public void leaveMethodInvocation(PMethodInvocation invo) {
        invo.typeCheck(currentLocals, space, currentClass);
    }
    
    public void leaveArrayCreation(PArrayCreation acre) {
        acre.resolveType(space, currentClass);
    }
    
    public void leaveArrayAccess(PArrayAccess access) {
        access.typeCheck(); 
    }
        
    public void leaveInstanceCreation(PInstanceCreation m) { 
        m.typeCheck(currentLocals, space, currentClass);    
    }
    
    public void leaveReturn(PReturn r) {
        r.setMethod(currentMethod);
    }
    
}
